#!/bin/bash

VERSION="1.1.8"

info() {
    echo -e "\033[32m[Info]\033[0m $1"
}

error() {
    echo -e "\033[31m[Error]\033[0m $1"
}

red() {
    echo -e "\033[31m$1\033[0m"
}

yellow() {
    echo -e "\033[0;33m$1\033[0m"
}

blue() {
    echo -e "\033[0;34m$1\033[0m"
}

# install_mosdns() {
#     if [ -f "/root/mosdns/mosdns" ]; then
#         cd "/root/mosdns"
#         chmod +x ./mosdns
#         ./mosdns service install
#     fi

# }

install_sbtool() {
    if [ -d "/root/sbtool" ]; then
        cd "/root/sbtool"
        chmod +x ./sbtool
        ./sbtool install
    fi
}

install_singbox() {
    cd "/root/singbox"
    cp sing-box /usr/local/bin/singbox
    chmod +x /usr/local/bin/singbox
    cd $install_dir
}

config_system() {
    cat >/etc/sysctl.conf <<EOF
vm.swappiness = 0
fs.file-max = 1024000
net.core.rmem_max = 134217728
net.core.wmem_max = 134217728
net.core.netdev_max_backlog = 250000
net.core.somaxconn = 1024000
net.core.default_qdisc = fq_pie
net.ipv4.conf.all.rp_filter = 0
net.ipv4.conf.default.rp_filter = 0
net.ipv4.conf.lo.arp_announce = 2
net.ipv4.conf.all.arp_announce = 2
net.ipv4.conf.default.arp_announce = 2
net.ipv4.ip_forward = 1
net.ipv4.ip_local_port_range = 1024 65535
net.ipv4.neigh.default.gc_stale_time = 120
net.ipv4.tcp_ecn = 0
net.ipv4.tcp_syncookies = 1
net.ipv4.tcp_tw_reuse = 1
net.ipv4.tcp_low_latency = 1
net.ipv4.tcp_fin_timeout = 10
net.ipv4.tcp_window_scaling = 1
net.ipv4.tcp_keepalive_time = 10
net.ipv4.tcp_timestamps = 0
net.ipv4.tcp_sack = 1
net.ipv4.tcp_fack = 1
net.ipv4.tcp_syn_retries = 3
net.ipv4.tcp_synack_retries = 3
net.ipv4.tcp_max_syn_backlog = 16384
net.ipv4.tcp_max_tw_buckets = 8192
net.ipv4.tcp_fastopen = 3
net.ipv4.tcp_mtu_probing = 1
net.ipv4.tcp_rmem = 4096 87380 67108864
net.ipv4.tcp_wmem = 4096 65536 67108864
net.ipv4.tcp_congestion_control = bbr
net.ipv6.conf.all.forwarding = 1
net.ipv6.conf.default.forwarding = 1
net.nf_conntrack_max = 25000000
net.netfilter.nf_conntrack_max = 25000000
net.netfilter.nf_conntrack_tcp_timeout_time_wait = 30
net.netfilter.nf_conntrack_tcp_timeout_established = 180
net.netfilter.nf_conntrack_tcp_timeout_close_wait = 30
net.netfilter.nf_conntrack_tcp_timeout_fin_wait = 30
EOF
    cat >/etc/security/limits.conf <<EOF
* soft nofile 512000
* hard nofile 512000
* soft nproc 512000
* hard nproc 512000
root soft nofile 512000
root hard nofile 512000
root soft nproc 512000
root hard nproc 512000
EOF

    default_interface=$(ip route | awk '/default/ {print $5}')
    cat >/etc/nftables.conf <<-EOF
#!/usr/sbin/nft -f

flush ruleset

table inet singbox {
  set local_ipv4 {
    type ipv4_addr
    flags interval
    elements = {
      10.0.0.0/8,
      127.0.0.0/8,
      169.254.0.0/16,
      172.16.0.0/12,
      192.168.0.0/16,
      240.0.0.0/4
    }
  }

  set local_ipv6 {
    type ipv6_addr
    flags interval
    elements = {
      ::ffff:0.0.0.0/96,
      64:ff9b::/96,
      100::/64,
      2001::/32,
      2001:10::/28,
      2001:20::/28,
      2001:db8::/32,
      2002::/16,
      fc00::/7,
      fe80::/10
    }
  }

  chain singbox-tproxy {
    fib daddr type { unspec, local, anycast, multicast } return
    ip daddr @local_ipv4 return
    ip6 daddr @local_ipv6 return
    udp dport { 123 } return
    meta l4proto { tcp, udp } meta mark set 1 tproxy to :9888 accept
  }

  chain singbox-mark {
    fib daddr type { unspec, local, anycast, multicast } return
    ip daddr @local_ipv4 return
    ip6 daddr @local_ipv6 return
    udp dport { 123 } return
    meta mark set 1
  }

  chain mangle-output {
    type route hook output priority mangle; policy accept;
    meta l4proto { tcp, udp } skgid != 1 ct direction original goto singbox-mark
  }

  chain mangle-prerouting {
    type filter hook prerouting priority mangle; policy accept;
    iifname {  lo, ${default_interface} } meta l4proto { tcp, udp } ct direction original goto singbox-tproxy
  }
}

EOF

    cat >/root/singbox/start.sh <<-EOF
set +e
/sbin/ip rule add fwmark 1 table 100
/sbin/ip route add local default dev lo table 100
/sbin/ip -6 rule add fwmark 1 table 101
/sbin/ip -6 route add local ::/0 dev lo table 101
set -e
/usr/local/bin/singbox run -D /root/singbox/
EOF
    chmod a+x /root/singbox/start.sh
    cat >/lib/systemd/system/singbox.service <<-EOF
[Unit]
Description=Singbox Default Server Service
Documentation=https://github.com/SagerNet/sing-box
After=network.target
# After=mosdns.service

[Service]
User=root
Type=simple
LimitNOFILE=32768
ExecStart=/bin/bash /root/singbox/start.sh

[Install]
WantedBy=multi-user.target
EOF
    # 关闭系统DNS监听53端口
    cat >>/etc/systemd/resolved.conf <<-EOF
DNSStubListener=no
EOF

    ln -sf /run/systemd/resolve/resolv.conf /etc/resolv.conf
    systemctl daemon-reload
    systemctl enable nftables
    systemctl enable singbox
    /usr/sbin/reboot
}

# cp -r ./mosdns /root/mosdns
if [ -d ./sbtool ]; then
    cp -r ./sbtool /root/sbtool
fi

cp -r ./singbox /root/singbox

# if [ -f /etc/systemd/system/mosdns.service ]; then
#     info "mosdns is already installed"
# else
#     install_mosdns
# fi

if [ -f /etc/systemd/system/sbtool.service ]; then
    info "sbtool is already installed"
else
    install_sbtool
fi

if [ -f /usr/local/bin/singbox ]; then
    info "singbox is already installed"
else
    install_singbox
fi

config_system
